import unittest
from unittest.mock import MagicMock, Mock, patch, call
from parameterized import parameterized
import pandas as pd
from pandas.testing import assert_frame_equal
from dgraphpandas.rdf import _resolve_transform, to_rdf
from dgraphpandas.strategies.horizontal import horizontal_transform
from dgraphpandas.strategies.vertical import vertical_transform


class RdfTests(unittest.TestCase):

    def test_resolve_transform_null_parameter(self):
        '''
        Ensures when the config is null
        an error is raised
        '''
        config = None
        with self.assertRaises(ValueError):
            _resolve_transform(config)

    def test_resolve_transform_horizontal(self):
        '''
        Ensures when transform is set to horizontal,
        then a horizontal is returned
        '''
        config = {'transform': 'horizontal'}
        transform_func = _resolve_transform(config)
        self.assertEqual(transform_func, horizontal_transform)

    def test_resolve_transform_vertical(self):
        '''
        Ensures when transform is set to vertical,
        then a vertical is returned
        '''
        config = {'transform': 'vertical'}
        transform_func = _resolve_transform(config)
        self.assertEqual(transform_func, vertical_transform)

    def test_resolve_transform_unknown(self):
        '''
        Ensures when the transform is unknown,
        then the default horizontal is used
        '''
        config = {'transform': 'something'}
        transform_func = _resolve_transform(config)
        self.assertEqual(transform_func, horizontal_transform)

    def test_resolve_transform_none_provided(self):
        '''
        Ensures when no transform is provided,
        then the default horizontal is used
        '''
        config = {}
        transform_func = _resolve_transform(config)
        self.assertEqual(transform_func, horizontal_transform)

    @parameterized.expand([
        (None, {'transform': 'horizontal'}, 'key'),
        (pd.DataFrame(), None, 'key'),
        (pd.DataFrame(), {'transform': 'horizontal'}, None)
    ])
    def test_to_rdf_null_parameters(self, frame, config, key):
        '''
        Ensures when parameters are null then an exception
        is thrown
        '''
        with self.assertRaises(ValueError):
            to_rdf(frame, config, key)

    @parameterized.expand([
        ('with_export_csv', {'export_csv': True}),
        ('with_export_rdf', {'export_rdf': True}),
    ])
    @patch('dgraphpandas.rdf.generate_upserts')
    @patch('dgraphpandas.rdf._get_config')
    @patch('dgraphpandas.rdf._resolve_transform')
    @patch('dgraphpandas.rdf.pd.read_csv')
    @patch('dgraphpandas.rdf.gzip.open')
    @patch('dgraphpandas.rdf.os.makedirs')
    def test_to_rdf_from_memory_frame_non_output_dir(
            self,
            name,
            options,
            mock_os_makedirs: Mock,
            mock_gzip: Mock,
            mock_pandas_read_csv_mock: Mock,
            mock_transform_func: Mock,
            mock_config: Mock,
            mock_upsert: Mock):
        '''
        Ensures when a memory frame is passed, it is passed through
        transformations and nothing is written to disk
        when output_dir is none even if the export params are set.
        '''
        frame = pd.DataFrame(data={
            'subject': [1, 2, 3],
            'predicate': ['age']*3,
            'object': [23, 43, 12]
        })
        config = {
            'transform': 'horizontal',
            'files': {'my_file': {
                'subject_fields': ['id']
            }}
        }
        config_key = 'my_file'

        mock_config.return_value = config
        mock_transform = MagicMock(return_value=(1, 2))
        mock_transform_func.return_value = mock_transform
        mock_upsert.return_value = (['intrinsic'], ['edges'])

        result = to_rdf(frame, config, config_key, output_dir=None, **(options))
        self.assertEqual(result, (['intrinsic'], ['edges']))

        args, kwargs = mock_config.call_args_list[0]
        self.assertEqual(args, (config,))
        self.assertEqual(kwargs, {})

        args, kwargs = mock_transform.call_args_list[0]
        passed_frame = args[0]
        passed_config = args[1]
        passed_key = args[2]

        assert_frame_equal(frame, passed_frame)
        self.assertEqual(config, passed_config)
        self.assertEqual(config_key, passed_key)
        self.assertEqual(kwargs, options)

        mock_pandas_read_csv_mock.assert_not_called()
        mock_gzip.assert_not_called()
        mock_os_makedirs.assert_not_called()

    @parameterized.expand([
        ###
        (
            'with_export_csv',
            [
                pd.DataFrame(data={
                    'subject': [1, 2, 3],
                    'predicate': ['age']*3,
                    'object': [23, 43, 12]
                })
            ],
            {
                'transform': 'horizontal',
                'files': {
                    'my_file': {
                        'subject_fields': ['id']
                    }
                }
            },
            'my_file',
            {'export_csv': True}
        ),
        ###
        (
            'with_export_rdf',
            [
                pd.DataFrame(data={
                    'subject': [1, 2, 3],
                    'predicate': ['age']*3,
                    'object': [23, 43, 12]
                })
            ],
            {
                'transform': 'horizontal',
                'files': {
                    'my_file': {
                        'subject_fields': ['id']
                    }
                }
            },
            'my_file',
            {'export_rdf': True}
        ),
        ###
        (
            'with_both_csv_and_rdf',
            [
                pd.DataFrame(data={
                    'subject': [1, 2, 3],
                    'predicate': ['age']*3,
                    'object': [23, 43, 12]
                })
            ],
            {
                'transform': 'horizontal',
                'files': {
                    'my_file': {
                        'subject_fields': ['id']
                    }
                }
            },
            'my_file',
            {'export_rdf': True, 'export_csv': True}
        ),
        ###
        (
            'with_console',
            [
                pd.DataFrame(data={
                    'subject': [1, 2, 3],
                    'predicate': ['age']*3,
                    'object': [23, 43, 12]
                })
            ],
            {
                'transform': 'horizontal',
                'files': {
                    'my_file': {
                        'subject_fields': ['id']
                    }
                }
            },
            'my_file',
            {'console': True}
        ),
        ###
        (
            'multiple_chunks',
            [
                pd.DataFrame(data={
                    'subject': [4, 5, 6],
                    'predicate': ['age']*3,
                    'object': [23, 43, 12]
                }),

                pd.DataFrame(data={
                    'subject': [1, 2, 3],
                    'predicate': ['age']*3,
                    'object': [23, 43, 12]
                })
            ],
            {
                'transform': 'horizontal',
                'files': {
                    'my_file': {
                        'subject_fields': ['id']
                    }
                }
            },
            'my_file',
            {'export_rdf': True, 'export_csv': True}
        ),
    ])
    @patch('dgraphpandas.rdf.generate_upserts')
    @patch('dgraphpandas.rdf._get_config')
    @patch('dgraphpandas.rdf._resolve_transform')
    @patch('dgraphpandas.rdf.pd.read_csv')
    @patch('dgraphpandas.rdf.gzip.open')
    @patch('dgraphpandas.rdf.os.makedirs')
    @patch('builtins.print')
    def test_to_rdf_from_file(
            self,
            name,
            frames,
            config,
            config_key,
            options,
            mock_print: Mock,
            mock_os_makedirs: Mock,
            mock_gzip: Mock,
            mock_pandas_read_csv_mock: Mock,
            mock_transform_func: Mock,
            mock_config: Mock,
            mock_upsert: Mock):
        '''
        Ensures when a file is provided, then transform is called
        '''
        mock_pandas_read_csv_mock.return_value = frames
        mock_config.return_value = config
        mock_intrinsic = MagicMock(spec=pd.DataFrame)
        mock_edges = MagicMock(spec=pd.DataFrame)
        mock_transform = MagicMock(return_value=(mock_intrinsic, mock_edges))
        mock_transform_func.return_value = mock_transform
        mock_upsert.return_value = (['intrinsic'], ['edges'])

        result = to_rdf('my_file.csv', config, config_key, output_dir='data/', **(options))
        self.assertEqual(result, [(['intrinsic'], ['edges'])]*len(frames))

        args, kwargs = mock_pandas_read_csv_mock.call_args_list[0]
        self.assertEqual(('my_file.csv',), args)
        self.assertEqual({'chunksize': 10_000_000}, kwargs)
        self.assertEqual(call('data/', exist_ok=True), mock_os_makedirs.call_args_list[0])

        if 'export_csv' in options:
            self.assertEqual(len(frames), len(mock_intrinsic.method_calls))
            self.assertEqual(len(frames), len(mock_edges.method_calls))
            self.assertEqual(mock_intrinsic.method_calls[0], call.to_csv('data/my_file_intrinsic.csv', encoding='utf-8', index=False))
            self.assertEqual(mock_edges.method_calls[0], call.to_csv('data/my_file_edges.csv', encoding='utf-8', index=False))
        else:
            self.assertEqual(0, len(mock_intrinsic.method_calls))
            self.assertEqual(0, len(mock_edges.method_calls))

        if 'export_rdf' in options:
            self.assertEqual(len(frames)*2, len(mock_gzip.call_args_list))
            self.assertEqual(call('data/my_file_intrinsic.gz', compresslevel=9, mode='wb'), mock_gzip.call_args_list[0])
            self.assertEqual(call('data/my_file_edges.gz', compresslevel=9, mode='wb'), mock_gzip.call_args_list[1])
        else:
            self.assertEqual(0, len(mock_gzip.call_args_list))

        if 'console' in options:
            mock_print.assert_called()

        mock_config.assert_called()
        mock_transform.assert_called()
        mock_transform_func.assert_called()
        mock_upsert.assert_called()



import unittest
from unittest.mock import patch, Mock

import pandas as pd
from pandas.testing import assert_frame_equal
from parameterized import parameterized

from dgraphpandas.strategies.horizontal import horizontal_transform


class HorizontalTests(unittest.TestCase):

    @parameterized.expand([
        (None, {'config': {}}, 'config_key'),
        (pd.DataFrame(), None, 'config_key'),
        (pd.DataFrame(), '', 'config_key'),
        (pd.DataFrame(), {'config': {}}, None),
        (pd.DataFrame(), {'config': {}}, ''),
    ])
    def test_horizontal_transform_null_parameters(self, frame, config, config_file_key):
        '''
        Ensures when parameters are null, then an
        error is raised
        '''
        with self.assertRaises(ValueError):
            horizontal_transform(frame, config, config_file_key)

    def test_horizontal_config_key_does_not_exist(self):
        '''
        Ensures when the config key does not exist
        within the config then an error is raised
        '''
        frame = pd.DataFrame()
        config_key = 'my_key'
        config = {
            'files': {
                'some_other_key': {}
            }
        }

        with self.assertRaises(KeyError):
            horizontal_transform(frame, config, config_key)

    @parameterized.expand([
        ('',),
        (None,),
    ])
    def test_horizontal_subject_fields_not_provided(self, subject_fields):
        '''
        Ensures when subject fields is not provided
        then an error is raised
        '''
        frame = pd.DataFrame()
        config_key = 'my_key'
        config = {
            'files': {
                'my_key': {
                    'subject_fields': subject_fields
                }
            }
        }

        with self.assertRaises(ValueError):
            horizontal_transform(frame, config, config_key)

    def test_horizontal_could_not_convert_type(self):
        '''
        Ensures when a type could not be applied to a column,
        then an error is raised
        '''
        frame = pd.DataFrame(data={
            'customer_id': [1, 2, 3],
            'age': [23, 'not number', 56]
        })
        config = {
            'files': {
                'customer': {
                    'subject_fields': ['customer_id'],
                    'type_overrides': {
                        'customer_id': 'int32',
                        'age': 'int32'
                    }
                }
            }
        }
        config_file_key = 'customer'
        with self.assertRaises(SystemExit):
            horizontal_transform(frame, config, config_file_key)

    @parameterized.expand([
        ###
        (
            'single_predicate',
            pd.DataFrame(data={
                'customer_id': [1, 2, 3],
                'age': [23, 67, 56]
            }),
            {
                'files': {
                    'customer': {
                        'subject_fields': ['customer_id'],
                        'type_overrides': {
                            'customer_id': 'int32',
                            'age': 'int32'
                        }
                    }
                }
            },
            'customer',
            pd.DataFrame(data={
                'customer_id': pd.Series([1, 2, 3], dtype='int32'),
                'predicate': pd.Series(['age']*3, dtype='O'),
                'object': pd.Series([23, 67, 56], dtype='int32')
            })
        ),
        ###
        (
            'multiple_predicates',
            pd.DataFrame(data={
                'customer_id': [1, 2, 3],
                'age': [23, 67, 56],
                'weight': [189, 167, 190]
            }),
            {
                'files': {
                    'customer': {
                        'subject_fields': ['customer_id'],
                        'type_overrides': {
                            'customer_id': 'int32',
                            'age': 'int32',
                            'weight': 'int32'
                        }
                    }
                }
            },
            'customer',
            pd.DataFrame(data={
                'customer_id': pd.Series([1, 2, 3, 1, 2, 3], dtype='int32'),
                'predicate': pd.Series(['age']*3 + ['weight']*3, dtype='O'),
                'object': pd.Series([23, 67, 56, 189, 167, 190], dtype='int32')
            })
        ),
        ###
        (
            'multiple_subject_fields',
            pd.DataFrame(data={
                'customer_id': [1, 2, 3],
                'order_id': [405, 210, 321],
                'value': [200, 321, 67],
            }),
            {
                'files': {
                    'order': {
                        'subject_fields': ['customer_id', 'order_id'],
                        'type_overrides': {
                            'customer_id': 'int32',
                            'order_id': 'int32',
                            'value': 'int32'
                        }
                    }
                }
            },
            'order',
            pd.DataFrame(data={
                'customer_id': pd.Series([1, 2, 3], dtype='int32'),
                'order_id': pd.Series([405, 210, 321], dtype='int32'),
                'predicate': pd.Series(['value']*3, dtype='O'),
                'object': pd.Series([200, 321, 67], dtype='int32')
            })
        )
    ])
    @patch('dgraphpandas.strategies.horizontal.vertical_transform')
    def test_horizontal_melted_passed(self, name, frame, config, config_file_key, expected_melted, transform_mock: Mock):
        '''
        Ensures that the passed horizontal frame is melted and
        passed into the vertical_transform.
        Also ensures the same config and key are passed through
        '''
        intrinsic_mock = Mock(spec=pd.DataFrame)
        edges_mock = Mock(spec=pd.DataFrame)
        transform_mock.return_value = (intrinsic_mock, edges_mock)

        intrinsic, edges = horizontal_transform(frame, config, config_file_key)

        transform_mock.assert_called_once()
        args, kwargs = transform_mock.call_args_list[0]
        invoked_frame, invoked_config, invoked_key = args

        assert_frame_equal(invoked_frame, expected_melted)
        self.assertEqual(invoked_config, config)
        self.assertEqual(invoked_key, config_file_key)
        self.assertEqual(kwargs, {})
        self.assertEqual(intrinsic_mock, intrinsic)
        self.assertEqual(edges_mock, edges)

    def test_horizontal_frame_only_has_subject_and_no_data_fields(self):
        '''
        Ensures when the horizontal frame only has subject fields
        and no actual data fields then an error is raised
        '''
        frame = pd.DataFrame(data={
            'customer_id': [1, 2, 3],
            'order_id': [405, 210, 321]
        })

        config = {
            'files': {
                'order': {
                    'subject_fields': ['customer_id', 'order_id'],
                    'type_overrides': {
                        'customer_id': 'int32',
                        'order_id': 'int32',
                    }
                }
            }
        }
        config_key = 'order'

        with self.assertRaises(ValueError):
            horizontal_transform(frame, config, config_key)

    @patch('dgraphpandas.strategies.horizontal.vertical_transform')
    @patch('dgraphpandas.strategies.horizontal.pd.read_csv', spec=pd.read_csv)
    def test_horizontal_melted_file_path_passed(self, mock_pandas: Mock, mock_transform: Mock):
        '''
        Ensures when a file path(str) it passed into the transform, then the file
        is read using read_csv before going into logic.
        '''
        file = 'test.csv'
        frame = pd.DataFrame(data={
            'customer_id': [1, 2, 3],
            'age': [23, 67, 56]
        })
        config = {
            'files': {
                'customer': {
                    'subject_fields': ['customer_id'],
                    'type_overrides': {
                        'customer_id': 'int32',
                        'age': 'int32'
                    }
                }
            }
        }
        config_file_key = 'customer'
        expected_melted = pd.DataFrame(data={
            'customer_id': pd.Series([1, 2, 3], dtype='int32'),
            'predicate': pd.Series(['age']*3, dtype='O'),
            'object': pd.Series([23, 67, 56], dtype='int32')

        })

        mock_pandas.return_value = frame

        horizontal_transform(file, config, config_file_key)

        args, kwargs = mock_pandas.call_args_list[0]
        self.assertEqual(file, args[0])

        args, kwargs = mock_transform.call_args_list[0]
        assert_frame_equal(expected_melted, args[0])
        self.assertEqual(config, args[1])
        self.assertEqual(config_file_key, args[2])

    @patch('dgraphpandas.strategies.horizontal.vertical_transform')
    @patch('dgraphpandas.strategies.horizontal.pd.read_csv', spec=pd.read_csv)
    def test_horizontal_melted_file_path_custom_csv_passed(self, mock_pandas: Mock, mock_transform: Mock):
        '''
        Ensures when a read_csv_options option is defined inside file configuration
        it is applied to the pd.read_csv call.
        '''
        file = 'test.csv'
        read_csv_options = {'sep': ';'}
        frame = pd.DataFrame(data={
            'customer_id': [1, 2, 3],
            'age': [23, 67, 56]
        })
        config = {
            'files': {
                'customer': {
                    'subject_fields': ['customer_id'],
                    'type_overrides': {
                        'customer_id': 'int32',
                        'age': 'int32'
                    },
                    'read_csv_options': read_csv_options
                }
            }
        }
        config_file_key = 'customer'
        expected_melted = pd.DataFrame(data={
            'customer_id': pd.Series([1, 2, 3], dtype='int32'),
            'predicate': pd.Series(['age']*3, dtype='O'),
            'object': pd.Series([23, 67, 56], dtype='int32')
        })

        mock_pandas.return_value = frame

        horizontal_transform(file, config, config_file_key)

        args, kwargs = mock_pandas.call_args_list[0]
        self.assertEqual(file, args[0])
        self.assertEqual(read_csv_options, kwargs)

        args, kwargs = mock_transform.call_args_list[0]
        assert_frame_equal(expected_melted, args[0])
        self.assertEqual(config, args[1])
        self.assertEqual(config_file_key, args[2])

    @parameterized.expand([
        ###
        (
            'year_wrong_order',
            {'dob': {'format': "%Y-%m-%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['03-02-2021', '01-03-1945'],
                'weight': [50, 32]
            })
        ),
        ###
        (
            'alphanumerical_string',
            {'dob': {'format': "%Y-%m-%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['not a date', '01-03-1945'],
                'weight': [50, 32]
            })
        ),
        ###
        (
            'missing_dashes',
            {'dob': {'format': "%Y-%m%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['2021-03-02', '19450301'],
                'weight': [50, 32]
            })
        ),
        ###
        (
            'missing_dots',
            {'dob': {'format': "%Y.%m.%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['2021-03-02', '1945.03&01'],
                'weight': [50, 32]
            })
        ),
        ###
        (
            'malformed_month_string',
            {'dob': {'format': "%d-%b-%Y"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['02-FebFake-2021', '01-Mar-1945'],
                'weight': [50, 32]
            })
        )
    ])
    @patch('dgraphpandas.strategies.horizontal.vertical_transform')
    def test_horizontal_transform_incorrect_date_format(self, name, date_format, frame, transform_mock: Mock):
        '''
        Ensures when the date format provided does not match the value within the frame,
        then an error is raised.
        '''
        config_file_key = 'customer'
        config = {
            'files': {
                config_file_key: {
                    'subject_fields': ['customer_id'],
                    'date_fields': date_format
                }
            }
        }

        with self.assertRaisesRegex(ValueError, "time data (.*) (doesn't|does not) match format(.*)"):
            horizontal_transform(frame, config, config_file_key)
        transform_mock.assert_not_called()

    @parameterized.expand([
        ###
        (
            'uncoverted_month_day',
            {'dob': {'format': "%Y"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['2021-03-02', '1945-03-01'],
                'weight': [50, 32]
            })
        ),
        ###
        (
            'uncoverted_month_year',
            {'dob': {'format': "%m-%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['03-02-2021', '03-01-2021'],
                'weight': [50, 32]
            })
        )
    ])
    @patch('dgraphpandas.strategies.horizontal.vertical_transform')
    def test_horizontal_transform_unconverted_date_parts(self, name, date_format, frame, transform_mock: Mock):
        '''
        Ensures when the date partially matches and there are some converted
        parts, an error is raised
        '''
        config_file_key = 'customer'
        config = {
            'files': {
                config_file_key: {
                    'subject_fields': ['customer_id'],
                    'date_fields': date_format
                }
            }
        }

        with self.assertRaisesRegex(ValueError, "unconverted data remains: (.*)"):
            horizontal_transform(frame, config, config_file_key)
        transform_mock.assert_not_called()

    @parameterized.expand([
        ###
        (
            'dash_format',
            {'dob': {'format': "%Y-%m-%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['2021-03-02', '1945-03-01'],
                'weight': [50, 32]
            }),
            pd.DataFrame(data={
                'customer_id': [1, 2, 1, 2],
                'predicate': ['dob', 'dob', 'weight', 'weight'],
                'object':[pd.to_datetime('2021-03-02 00:00:00'), pd.to_datetime('1945-03-01 00:00:00'), 50, 32]
            })
        ),
        ###
        (
            'dot_format',
            {'dob': {'format': "%Y.%m.%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['1999.05.09', '1789.02.12'],
                'weight': [50, 32]
            }),
            pd.DataFrame(data={
                'customer_id': [1, 2, 1, 2],
                'predicate': ['dob', 'dob', 'weight', 'weight'],
                'object': [pd.to_datetime('1999-05-09 00:00:00'), pd.to_datetime('1789-02-12 00:00:00'), 50, 32]
            })
        ),
        ###
        (
            'multiple_date_fields',
            {'updated_at': {'format': '%Y.%m.%d'}, 'dob': {'format': "%Y.%m.%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['1999.05.09', '1789.02.12'],
                'updated_at': ['2021.03.02', '2021.03.04'],
                'weight': [50, 32]
            }),
            pd.DataFrame(data={
                'customer_id': [1, 2, 1, 2, 1, 2],
                'predicate': ['dob', 'dob', 'updated_at', 'updated_at', 'weight', 'weight'],
                'object': [
                    pd.to_datetime('1999-05-09 00:00:00'),
                    pd.to_datetime('1789-02-12 00:00:00'),
                    pd.to_datetime('2021-03-02 00:00:00'),
                    pd.to_datetime('2021-03-04 00:00:00'),
                    50,
                    32]
            })
        ),
        ###
        (
            'multiple_date_fields_different_formats',
            {'updated_at': {'format': '%Y$%m$%d'}, 'dob': {'format': "%Y.%m.%d"}},
            pd.DataFrame(data={
                'customer_id': [1, 2],
                'dob': ['1999.05.09', '1789.02.12'],
                'updated_at': ['2021$03$02', '2021$03$04'],
                'weight': [50, 32]
            }),
            pd.DataFrame(data={
                'customer_id': [1, 2, 1, 2, 1, 2],
                'predicate': ['dob', 'dob', 'updated_at', 'updated_at', 'weight', 'weight'],
                'object': [
                    pd.to_datetime('1999-05-09 00:00:00'),
                    pd.to_datetime('1789-02-12 00:00:00'),
                    pd.to_datetime('2021-03-02 00:00:00'),
                    pd.to_datetime('2021-03-04 00:00:00'),
                    50,
                    32]
            })
        )
    ])
    @patch('dgraphpandas.strategies.horizontal.vertical_transform')
    def test_horizontal_transform_correct_date_format(self, name, date_format, frame, expected_melted, transform_mock: Mock):
        '''
        Ensures when the date_format provided is in the correct format,
        no error is raised
        '''
        config_file_key = 'customer'
        config = {
            'files': {
                config_file_key: {
                    'subject_fields': ['customer_id'],
                    'date_fields': date_format
                }
            }
        }

        horizontal_transform(frame, config, config_file_key)

        transform_mock.assert_called_once()
        args, kwargs = transform_mock.call_args_list[0]

        passed_frame, passed_config, passed_config_key = args

        assert_frame_equal(passed_frame, expected_melted)
        self.assertEqual(passed_config, config)
        self.assertEqual(passed_config_key, config_file_key)
        self.assertEqual(kwargs, {})



import os
import logging
import gzip
from typing import Any, Dict, Union, Tuple, List, Callable

import pandas as pd

from dgraphpandas.config import get_from_config, _get_config
from dgraphpandas.writers.upserts import generate_upserts
from dgraphpandas.strategies.vertical import vertical_transform
from dgraphpandas.strategies.horizontal import horizontal_transform

logger = logging.getLogger(__name__)


def _resolve_transform(config: Dict[str, Any]):
    '''
    Based on the transform configuration, choose
    a transform function
    '''
    if config is None:
        raise ValueError('config')
    if 'transform' not in config:
        return horizontal_transform

    if config['transform'] == 'horizontal':
        transform_func = horizontal_transform
    elif config['transform'] == 'vertical':
        transform_func = vertical_transform
    else:
        logger.debug('Transform not set within configuration, defaulting to horizontal')
        transform_func = horizontal_transform

    return transform_func


def to_rdf(
        frame: Union[str, pd.DataFrame],
        config: Union[Dict[str, Any], str],
        config_key: str,
        output_dir: Union[str, None] = None,
        **kwargs) -> Union[None, List[Tuple[List[str], List[str]]]]:
    '''
    Converts a Pandas DataFrame into RDF Exports.

    Parameters:
        frame: A Pandas DataFrame or file path to a CSV to be converted.
        config: A Configuration Dictionary or file path
        config_key: The file (key) to use in the configuration
        output_dir: The output directory to push exports. If none, don't export

    Returns:
        If chunking was applied then a list of tuples
        If no chunking then just a tuple
        Each tuple has two items: intrinsic and edges
    '''
    if frame is None:
        raise ValueError('frame')
    if not config:
        raise ValueError('config')
    if not config_key:
        raise ValueError('config_key')

    config = _get_config(config)
    transform_func = _resolve_transform(config)

    '''
    The Frame may be a file path or already loaded DataFrame.
    If it's a string then attempt to load the file.
    '''
    if isinstance(frame, str):
        file_config = config['files'][config_key]
        read_csv_options: Dict[str, Any] = get_from_config('read_csv_options', file_config, {}, **(kwargs))
        chunk_size: int = get_from_config('chunk_size', config, 10_000_000, **(kwargs))
        source_file_name = os.path.basename(frame).split('.')[0]

        result = []
        for index, frame in enumerate(pd.read_csv(frame, chunksize=chunk_size, **(read_csv_options))):
            result.append(to_rdf_from_frame(frame, config, config_key, transform_func, source_file_name, output_dir, index, **(kwargs)))
        return result
    else:
        return to_rdf_from_frame(frame, config, config_key, transform_func, config_key, output_dir, 0, **(kwargs))


def to_rdf_from_frame(
        frame: pd.DataFrame,
        config: Dict[str, Any],
        config_key,
        transform_func: Callable,
        source_file_name: str,
        output_dir: str,
        index: int = 0,
        **kwargs):

    file_config = config['files'][config_key]
    console: bool = get_from_config('console', config, False, **(kwargs))
    export_csv: bool = get_from_config('export_csv', file_config, False, **(kwargs))
    export_rdf: bool = get_from_config('export_rdf', file_config, False, **(kwargs))
    encoding: str = get_from_config('encoding', file_config, 'utf-8', **(kwargs))
    gz_compression_level: int = get_from_config('gz_compression_level', file_config, 9, **(kwargs))

    logger.info('Transforming Source Frame to Rdf Frame')
    intrinsic, edges = transform_func(frame, config, config_key, **(kwargs))
    if console:
        print('Intrinsic \n', intrinsic)
        print('Edges \n', edges)

    intrinsic_upserts, edges_upserts = generate_upserts(intrinsic, edges)
    if output_dir is not None:
        os.makedirs(output_dir, exist_ok=True)
        if index == 0:
            intrinsic_base_path = os.path.join(output_dir, source_file_name + '_intrinsic')
            edges_base_path = os.path.join(output_dir, source_file_name + '_edges')
        else:
            intrinsic_base_path = os.path.join(output_dir, source_file_name + '_intrinsic_' + str(index+1))
            edges_base_path = os.path.join(output_dir, source_file_name + '_edges_' + str(index+1))

        if export_csv:
            intrinsic_csv_path = intrinsic_base_path + '.csv'
            edges_csv_path = edges_base_path + '.csv'

            logger.info(f'Writing to {intrinsic_csv_path}')
            intrinsic.to_csv(intrinsic_csv_path, index=False, encoding=encoding)

            logger.info(f'Writing to {edges_csv_path}')
            edges.to_csv(edges_csv_path, index=False, encoding=encoding)

        if export_rdf:
            logger.info('Generating Rdf Upserts from Frames')

            intrinsic_gz_path = intrinsic_base_path + '.gz'
            logger.info(f'Writing to {len(intrinsic_upserts)} upserts to {intrinsic_gz_path}')
            with gzip.open(intrinsic_gz_path, mode='wb', compresslevel=gz_compression_level) as zip:
                s = '\n'.join(intrinsic_upserts)
                s = s.encode(encoding=encoding)
                zip.write(s)

            edges_gz_path = edges_base_path + '.gz'
            logger.info(f'Writing to {len(edges_upserts)} upserts to {edges_gz_path}')
            with gzip.open(edges_gz_path, mode='wb', compresslevel=gz_compression_level) as zip:
                s = '\n'.join(edges_upserts)
                s = s.encode(encoding=encoding)
                zip.write(s)

    return intrinsic_upserts, edges_upserts



import unittest
from unittest.mock import patch, Mock, mock_open
from parameterized import parameterized
from dgraphpandas.config import get_from_config, _get_config


class ConfigTests(unittest.TestCase):

    @parameterized.expand([
        (None, {'hello': 'world'}, 'default', {'firstkey': 'secondkey'}),
        ('key', None, 'default', {'firstkey': 'secondkey'})
    ])
    def test_get_from_config_nullkey_exception(
            self,
            key,
            config,
            default,
            kwargs):
        '''
        Ensures when the passed key is none, then an exception is raised.
        '''
        with self.assertRaises(ValueError):
            get_from_config(key, config, default, **(kwargs))

    def test_get_from_config_override_exists(self):
        '''
        Ensures when an override exists in kwargs, then it is used
        even if a value exists in config
        '''
        key = 'subject_fields'
        config = {'subject_fields': 'id'}
        value = get_from_config(key, config, subject_fields='override')

        self.assertEqual(value, 'override')

    def test_get_from_config_no_override_exists(self):
        '''
        Ensures when no override exists in kwargs, then the config value is used
        '''
        key = 'subject_fields'
        config = {'subject_fields': 'id'}
        value = get_from_config(key, config, something_else='override')

        self.assertEqual(value, 'id')

    def test_get_from_config_no_override_and_no_config_default(self):
        '''
        Ensures when no override exists in kwargs, then the config value is used
        '''
        key = 'subject_fields'
        config = {'some_other_field': 'value'}
        value = get_from_config(key, config, something_else='override', default='default_value')

        self.assertEqual(value, 'default_value')

    def test_get_config_null_config(self):
        '''
        Ensures when the config is null
        an exception is raised
        '''
        config = None
        with self.assertRaises(ValueError):
            _get_config(config)

    @patch('builtins.open', callable=mock_open)
    @patch('dgraphpandas.config.json.load')
    def test_get_config_file(self, mock_json: Mock, mock_open: Mock):
        '''
        Ensures when a file (str) is passed, open is called and
        a json is loaded
        '''
        config = 'my_config'
        expected_config = {'my_config': 'hello'}
        mock_json.return_value = expected_config

        result = _get_config(config)

        self.assertEqual(expected_config, result)

        args, kwargs = mock_open.call_args_list[0]
        self.assertEqual(args, (config, 'r'))
        self.assertEqual(kwargs, {})

    def test_get_config_passed(self):
        '''
        Ensures when an actual config is passed,
        it is returned back
        '''
        config = {'my_config': 'hello'}
        returned = _get_config(config)
        self.assertEqual(config, returned)



import logging
import argparse
import os
import json
from typing import Any, Dict
from pprint import pformat

import pandas as pd

from dgraphpandas import __version__, __description__, to_rdf

pd.set_option('mode.chained_assignment', None)


def main():
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument('-f', '--file', required=True, help='The Data File (CSV) to convert into RDF.')
    parser.add_argument('-c', '--config', required=True, help='The DgraphPandas Configuration. See Documentation for options/examples.')
    parser.add_argument('-ck', '--config_file_key', required=True, help='The Entry in the Configuration to use for this passed file.')
    parser.add_argument('-o', '--output_dir', default='.', help='The output directory to write files.')
    parser.add_argument('--console', action='store_true', default=False, help='Write the Preprocessed DataFrames to console (for debugging)')
    parser.add_argument('--export_csv', action='store_true', default=False, help='Write the Preprocessed DataFrame to CSV (for debugging)')
    parser.add_argument('--encoding', default=os.environ.get('DGRAPHPANDAS_ENCODING', 'utf-8'), help='The Encoding to write files.')
    parser.add_argument('--chunk_size', default=10_000_000, type=int, help='Process and output in chunks rather all at once')
    parser.add_argument('--gz_compression_level', default=9, type=int, help='Compression level to set output gzip files to')

    parser.add_argument('--key_separator')
    parser.add_argument('--add_dgraph_type_records', default=True)
    parser.add_argument('--drop_na_intrinsic_objects', default=True)
    parser.add_argument('--drop_na_edge_objects', default=True)
    parser.add_argument('--illegal_characters', default=['%', '\\.', '\\s', '\"', '\\n', '\\r\\n'])
    parser.add_argument('--illegal_characters_intrinsic_object', default=['\"', '\\n', '\\r\\n'])
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('-v', '--verbosity',
                        choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'NOTSET'],
                        default=os.environ.get('DGRAPHPANDAS_LOG', 'INFO'))

    args = parser.parse_args()

    logging.basicConfig(level=args.verbosity)
    logger = logging.getLogger(__name__)

    try:
        import coloredlogs
        coloredlogs.install(level=args.verbosity)
    except ImportError as e:
        logger.debug(e)

    options = {
        'key_separator': args.key_separator,
        'add_dgraph_type_records': args.add_dgraph_type_records,
        'drop_na_intrinsic_objects': args.drop_na_intrinsic_objects,
        'drop_na_edge_objects': args.drop_na_edge_objects,
        'illegal_characters': args.illegal_characters,
        'illegal_characters_intrinsic_object': args.illegal_characters_intrinsic_object,
        'console': args.console,
        'export_csv': args.export_csv,
        'chunk_size': args.chunk_size
    }
    options = {key: value for key, value in options.items() if value is not None and value is not False}

    with open(args.config, 'r') as f:
        global_config: Dict[str, Any] = json.load(f)
        logger.debug('Global Config \n %s', pformat(global_config))

    to_rdf(args.file, global_config, args.config_file_key, args.output_dir, export_rdf=True, **(options))


if __name__ == '__main__':
    main()



import json
import logging
from pprint import pformat
from typing import Dict, Any, Union

logger = logging.getLogger(__name__)


def get_from_config(key: str, config: Dict[str, Any], default: Any = None, **kwargs) -> Any:
    '''
    For the given configuration key,
    first try to pull from kwargs (if there is an override)
    and secondly pull from the passed config dictionary
    else return default
    '''
    if not key:
        raise ValueError('key')
    if not config:
        raise ValueError('config')

    return kwargs.get(key, config.get(key, default))


def _get_config(config: Union[str, Dict[str, Any]]) -> Dict[str, Any]:
    '''
    The Configuration may be a file path or already loaded config.
    If it's a string then attempt to load it first.
    '''
    if config is None:
        raise ValueError('config')

    if isinstance(config, str):
        logger.info(f'Reading from configuration {config}')
        with open(config, 'r') as f:
            config: Dict[str, Any] = json.load(f)
            logger.debug('Global Config \n %s', pformat(config))
        return config
    else:
        return config



from setuptools import setup, find_packages
from dgraphpandas import __name__, __version__, __description__

with open('README.md', 'r') as desc:
    long_description = desc.read()

with open('requirements.txt', 'r') as f:
    requirements = f.read()
    requirements = requirements.split('\n')

setup(
    name=__name__,
    version=__version__,
    description=__description__,
    packages=find_packages(),
    install_requires=requirements,
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/kiran94/dgraphpandas',
    entry_points={
        'console_scripts': [
            'dgraphpandas = dgraphpandas.__main__:main'
        ]
    },
    keywords=['dgraph', 'pandas', 'rdf', 'graph', 'database'],
    python_requires='>=3.6',
    license='MIT'
)
